;;;; -*- lisp -*-
;;;; $Id: defpackage.lisp,v 1.4 2007/07/13 10:00:42 willijar Exp $
;;;; Copyright (C) 2003 Dr. John A.R. Williams, J.A.R.Williams@blueyonder.co.uk

;;;; This file is part of the Common Lisp Markup Package

;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.

;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.

;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
;;;; or retreive a copy from http://www.gnu.org/licenses/gpl.txt

;;;; Package definition for markup handling

(in-package :cl-user)

(defpackage :markup
  (:documentation
   "Package inside which markup symbols and processors are kept")
  (:use :cl)
  (:import-from :jarw.media
                #:register-media #:media-variant #:media-variant-url
                #:*media-server* #:media-variant-path)
  (:import-from :jarw.lib #:when-bind #:while)
  (:import-from :jarw.string #:expand-tabs #:strcat #:join-strings)
  (:import-from :regex #:compile-str #:scan-str
                #:regex-process-string #:regex-split-string)
  (:import-from :jarw.io
                #:log-stream #:write-log #:find-file #:*search-path*
                #:with-temp-pathname #:copy-stream )
  (:import-from :jarw.search #:keywords #:ignore-word-p)
  (:import-from :jarw.parse #:date #:format-output)
  (:export
   ;; global symbols that control output
   #:*search-path* #:*section-level* #:*document-name
   ;; markup element tags declared in this package - first html like tags
   #:head #:title #:tableofcontents #:toc #:author #:date #:document
   #:body #:p #:table #:tr #:th #:td #:math #:em #:strong #:br
   #:select #:meta #:code #:pre #:b #:img
   #:cite #:form #:input #:textarea  #:ol #:ul #:dl #:li #:dt #:dd #:radio
   #:h1 #:h2 #:h3 #:h4 #:h5 #:h6 #:h7 #:h8 #:div #:input #:a #:quote
   ;; invented tags with programmed behaviour
   #:figure #:file #:equation #:section #:ref #:graphic
   #:mcq #:maq #:navbar #:boolean #:structured-text #:cite
   #:footnote #:footnotes
   ;; now the functionality
   #:split-markup #:join-markup #:with-markup-environment
   #:substitute-markup-when #:process-markup-when #:markup-keywords
   #:markup-scanner #:markup-transformation #:markup-tag #:markup-attr
   #:markup-content
   #:remove-attribute #:remove-attribute-if
   #:with-new-document
   #:html #:preprocess-html #:latex #:strip-markup #:docbook
   #:parse-structured-text  #:pdf #:with-structured-text-syntax))

(defpackage :structured-text
  (:use :cl :markup)
  (:import-from :jarw.string #:split-string)
  (:export #:parse-structured-text #:structured-text
	   #:*tab-size*))
