;;;; -*- lisp -*-
;;;; $Id: html-utilities.lisp,v 1.1 2006/07/18 18:51:57 willijar Exp $
;;;; Copyright (C) 2002 Dr. John A.R. Williams, J.A.R.Williams@blueyonder.co.uk

;;;; This file is part of the Common Lisp Markup Package

;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.

;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.

;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
;;;; or retreive a copy from http://www.gnu.org/licenses/gpl.txt

(in-package :markup)

(defvar *html-quote-map* '((#\< . "&lt;") (#\> . "&gt;")
			   (#\& . "&amp;") (#\" . "&quot;"))
  "An association list mapping characters to there html-quoted form
   Do not modify this list directly,
   Use the function set-html-quote-map to modify this list ")

(let ((html-quote-matcher))
  (defun set-html-quote-map (quote-map)
    "Sets *html-quote-map*  to a new value, and compiles matcher
   as a side effect"
    (setq *html-quote-map* quote-map)
    (setf html-quote-matcher
 	  (compile-str
	   (concatenate 'string
			"["
			(make-array (length quote-map)
				    :element-type 'character
				    :initial-contents (mapcar #'car quote-map))
			"]"))))
  (set-html-quote-map *html-quote-map*)
  (defun html-quote (html-string)
    (apply #'concatenate
	   (cons 'string
		 (regex-process-string html-quote-matcher html-string
				   #'(lambda(regs)
				       (cdr (assoc (char (elt regs 0) 0)
						   *html-quote-map*))))))))

(defun url-quote(stream string)
  (loop for char across string
	do (if (or (alphanumericp char) (find char "$-_.+!*'(),"))
	    (write-char char stream)
	    (format stream "%~X" (char-code char)))))

(defun html-query-string(alist)
  "Construct a query string from an alist - inverse of url-query-param"
  (let ((alist (mapcan #'(lambda(item) (when (cdr item) (list item))) alist)))
  (with-output-to-string (s)
    (flet ((encode(assoc)
	     (princ (car assoc) s)
	     (write-char #\= s)
	     (url-quote s (princ-to-string (cdr assoc)))))
      (encode (first alist))
      (dolist (item alist)
	(write-char #\& s)
	(encode item))))))

;; an example of using state machines provided on usenet by
;--
;Raymond Wiker                        Mail:  Raymond.Wiker@fast.no
;Senior Software Engineer             Web:   http://www.fast.no/
;Fast Search & Transfer ASA           Phone: +47 23 01 11 60
;P.O. Box 1677 Vika                   Fax:   +47 35 54 87 99
;NO-0120 Oslo, NORWAY                 Mob:   +47 48 01 11 60
;
;Try FAST Search: http://alltheweb.com/
;----------

(defparameter *strip-xml-parse-table*
  #(((#\< . 1) 0)			; 0 - normal state
    ((#\! . 2) 5)			; 1 - after <
    ((#\- . 3) 5)			; 2 - after <!
    ((#\- . 4) 5)			; 3 - after <!-
    ((#\- . 8) 4)			; 4 - comment <!--
    ((#\' . 6)				; 5 - markup
      (#\" . 7)
      (#\> . 0)
      5)
    ((#\' . 5) 6)			; 6 - markup, single-quote
    ((#\" . 5) 7)			; 7 - markup, double-quote
    ((#\- . 9) 4)			; 8 - comment, after -
    ((#\> . 0) (#\- . 9) 4)		; 9 - comment, after --(*)
    ))

(defun lookup-transition (state input)
  (let ((transitions (svref *strip-xml-parse-table* state)))
    (dolist (elt transitions)
      (cond ((atom elt)
	     (return-from lookup-transition elt))
	    ((char-equal input (car elt))
	     (return-from lookup-transition (cdr elt)))))
    (error "State table error")))

(defun strip-xml-markup (string)
  (let ((state 0))
    (with-output-to-string (out)
      (with-input-from-string (in string)
	(do ((char (read-char in nil) (read-char in nil)))
	    ((null char))
	  (let ((new-state (lookup-transition state char)))
	    (when (zerop new-state)
	      (write-char
	       (if (zerop state)
		   char
		   #\Space)
	       out))
	    (setf state new-state)))))))
