;;;; -*- lisp -*-
;;;; $Id: strip.lisp,v 1.1 2006/07/18 18:52:43 willijar Exp $
;;;; Copyright (C) 2002 Dr. John A.R. Williams, J.A.R.Williams@blueyonder.co.uk

;;;; This file is part of the Common Lisp Markup Package

;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.

;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.

;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
;;;; or retreive a copy from http://www.gnu.org/licenses/gpl.txt


(in-package :markup)

(defgeneric strip-markup(stream tag &optional attr content)
  (:documentation "Strips markup leaving only text - useful for searches"))

(defmethod strip-markup((stream (eql nil)) markup &optional attr content)
  (with-output-to-string(s) (strip-markup s markup attr content)))

(defmethod strip-markup(stream tag &optional attr content)
  "Default - just print it"
  (declare (ignore content attr))
  (princ tag stream))

(defmethod strip-markup(stream (markup list) &optional attr content)
  (declare (ignore content attr))
  (apply #'strip-markup
	 (cons stream (multiple-value-list (split-markup markup)))))

(defmethod strip-markup(stream (tag symbol) &optional attr content)
  (when (getf attr :title)
    (princ (getf attr :title) stream)
    (write-char #\space stream))
  (unless (empty-element-p tag)
    (dolist (item content) (strip-markup stream item))
    (write-char #\space stream)))

